@@ -9,9 +9,9 @@
 #include <string>
 #include <vector>
 #include <Rcpp.h>
-#include <boost/optional.hpp>
-#include <boost/date_time.hpp>
+#include "optional.h"
 #include "thread.h"
+#include "timegm.h"
 
 // A callback for deleting objects on the main thread using later(). This is
 // needed when the object is an Rcpp object or contains one, because deleting
@@ -56,14 +56,15 @@ inline void err_printf(const char *fmt, ...) {
     return;
 
   ssize_t res = write(STDERR_FILENO, buf, n);
-  // This is here simply to avoid a warning about "ignoring return value" of
-  // the write(), on some compilers. (Seen with gcc 4.4.7 on RHEL 6)
-  res += 0;
+  // This is here simply to avoid a warning about "ignoring return value" of the
+  // write(), or "variable 'res' set but not used" on some compilers.
+  if (res) res += 0;
+  return;
 }
 
 
 // ============================================================================
-// Logging 
+// Logging
 // ============================================================================
 
 enum LogLevel {
@@ -134,26 +135,26 @@ std::map<std::string, T1> toMap(T2 x) {
   return strmap;
 }
 
-// A wrapper for Rcpp::as. If the R value is NULL, this returns boost::none;
+// A wrapper for Rcpp::as. If the R value is NULL, this returns nullopt;
 // otherwise it returns the usual value that Rcpp::as returns, wrapped in
-// boost::optional<T2>.
+// std::experimental::optional<T2>.
 template <typename T1, typename T2>
-boost::optional<T1> optional_as(T2 value) {
+std::experimental::optional<T1> optional_as(T2 value) {
   if (value.isNULL()) {
-    return boost::none;
+    return std::experimental::nullopt;
   }
-  return boost::optional<T1>( Rcpp::as<T1>(value) );
+  return std::experimental::optional<T1>( Rcpp::as<T1>(value) );
 }
 
-// A wrapper for Rcpp::wrap. If the C++ value is boost::none, this returns the
+// A wrapper for Rcpp::wrap. If the C++ value is missing, this returns the
 // R value NULL; otherwise it returns the usual value that Rcpp::wrap returns, after
-// unwrapping from the boost::optional<T>.
+// unwrapping from the std::experimental::optional<T>.
 template <typename T>
-Rcpp::RObject optional_wrap(boost::optional<T> value) {
-  if (value == boost::none) {
+Rcpp::RObject optional_wrap(std::experimental::optional<T> value) {
+  if (!value.has_value()) {
     return R_NilValue;
   }
-  return Rcpp::wrap(value.get());
+  return Rcpp::wrap(*value);
 }
 
 
@@ -261,66 +262,9 @@ inline std::string http_date_string(const time_t& t) {
   return std::string(res);
 }
 
-
 // Given a date string of format "Wed, 21 Oct 2015 07:28:00 GMT", return a
 // time_t representing that time. If the date is malformed, then return 0.
-inline time_t parse_http_date_string(const std::string& date) {
-  // This is because the static std::locale may not be thread-safe. If in the
-  // future we need to call this from multiple threads, we can remove this and
-  // make the std::locale non-static.
-  ASSERT_BACKGROUND_THREAD()
-
-  if (date.length() != 29) {
-    return 0;
-  }
-
-  // Strip off leading "Wed, ". We'll just ignore it.
-  std::string date_str = date.substr(5);
-
-  // Make sure it ends with " GMT", then strip it off.
-  if (date_str.substr(date_str.size() - 4, 4) != " GMT") {
-    return 0;
-  }
-  date_str = date_str.substr(0, date_str.size() - 4);
-
-  // Format is now "21 Oct 2015 07:28:00". Next, replace month with number, so
-  // that there are no locale issues parsing date.
-  std::string month_names[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
-                                "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
-  std::string month_name = date_str.substr(3, 3);
-  int month_num = 0;
-  for (int i=0; i<12; i++) {
-    if (month_name == month_names[i]) {
-      month_num = i + 1;
-      break;
-    }
-  }
-
-  if (month_num == 0) {
-    return 0;
-  }
-
-  date_str.replace(3, 3, toString(month_num));
-
-  // Format is now something like "21 10 2015 07:28:00"
-  // Convert to a boost::posix_time::ptime object
-  const static std::locale time_format = std::locale(
-    std::locale::classic(),
-    // Apparently it's not necessary to delete the time_input_facet.
-    new boost::posix_time::time_input_facet("%d %m %Y %H:%M:%S")
-  );
-
-  boost::posix_time::ptime pt, ptbase;
-  std::istringstream is(date_str);
-  is.imbue(time_format);
-  is >> pt;
-  if (pt == ptbase) {
-    // Error parsing time
-    return 0;
-  }
-
-  return to_time_t(pt);
-}
+time_t parse_http_date_string(const std::string& date);
 
 // Compares two strings in constant time. Returns true if they are the same;
 // false otherwise.
